@@ -52,7 +52,7 @@ subtype PkgName of Str where { .defined and m/^<[a-zA-Z0-9_:]>+$/ }
 ######################################################################
 ######################################################################
 
-module Locale::KeyedText-0.0.2 {
+module Locale::KeyedText-0.0.3 {
 
 ######################################################################
 
@@ -180,147 +180,84 @@ method as_string( $translator: ) returns Str {
 
 1;
 
-=pod
-
 =head1 SYNOPSIS
 
-The following demonstrates a simple library and a simple program that uses it;
-both are N-multi-lingual and ship with English and French support files.  While 
-there is no support file specific to the library for a certain third language, 
-the one with the program also adds support to the library.
+	require Locale::KeyedText;
 
-=head2 Content of shared library file 'MyLib.pm':
+	main();
+
+	sub main() {
+		# Create a translator.
+		my $translator = Locale::KeyedText.new_translator( 
+			['MyLib::Lang::', 'MyApp::Lang::'],  # set package prefixes for localized app components
+			['Eng', 'Fr', 'De', 'Esp']           # set list of available languages in order of preference
+		);
+
+		# This will print 'Enter 2 Numbers' in the first of the four languages that has a matching template available.
+		print $translator.translate_message( Locale::KeyedText.new_message( 'MYAPP_PROMPT' ) );
+
+		# Read two numbers from the user.
+		my Num ($first, $second) = $*IN;
+
+		# Print a statement giving the operands and their sum.
+		MyLib.add_two( $first, $second, $translator );
+	}
 
 	module MyLib;
 
-	require Locale::KeyedText;
+	sub add_two( Num $first, Num $second, Locale::KeyedText::Translator $translator ) {
+		my Num $sum = $first + $second;
 
-	sub my_invert( Str $number ) returns Num {
-		$number.defined or throw Locale::KeyedText.new_message( 'MYLIB_MYINV_NO_ARG' );
-		$number ~~ m/\d/ or throw Locale::KeyedText.new_message( 
-			'MYLIB_MYINV_BAD_ARG', { 'GIVEN_VALUE' => $number } );
-		$number == 0 and throw Locale::KeyedText.new_message( 'MYLIB_MYINV_RES_INF' );
-		return 1 / $number;
+		# This will print '<FIRST> plus <SECOND> equals <RESULT>' in the first possible language.
+		# For example, if the user inputs '3' and '4', it the output will be '3 plus 4 equals 7'.
+		print $translator->translate_message( Locale::KeyedText->new_message( 'MYLIB_RESULT', 
+			{ 'FIRST' => $first, 'SECOND' => $second, 'RESULT' => $sum } ) );
 	}
 
-=head2 Content of English language Template file 'MyLib/L/Eng.pm':
-
-	module MyLib::L::Eng;
-
-	my Str %text_strings ::= (
-		'MYLIB_MYINV_NO_ARG' => 'my_invert(): argument NUMBER is missing',
-		'MYLIB_MYINV_BAD_ARG' => 'my_invert(): argument NUMBER is not a number, it is "{GIVEN_VALUE}"',
-		'MYLIB_MYINV_RES_INF' => 'my_invert(): result is infinite because argument NUMBER is zero',
-	);
-
-	sub get_text_by_key( Str $msg_key ) returns Str { return $text_strings{$msg_key}; }
-
-=head2 Content of French language (rough manual translation) Template file 'MyLib/L/Fre.pm':
-
-	module MyLib::L::Fre;
-
-	my Str %text_strings ::= (
-		'MYLIB_MYINV_NO_ARG' => 'my_invert(): paramètre NUMBER est manquant',
-		'MYLIB_MYINV_BAD_ARG' => 'my_invert(): paramètre NUMBER est ne nombre, il est "{GIVEN_VALUE}"',
-		'MYLIB_MYINV_RES_INF' => 'my_invert(): aboutir a est infini parce que paramètre NUMBER est zero',
-	);
-
-	sub get_text_by_key( Str $msg_key ) returns Str { return $text_strings{$msg_key}; }
-
-=head2 Content of main program 'MyApp.pl':
-
-	require MyLib;
-	require Locale::KeyedText;
-
-	main( grep { $_ ~~ m/^<[a-zA-Z]>+$/ } @*ARGS ); # user indicates language as command line argument
-
-	sub main( Str ?@user_lang_prefs = 'Eng' ) {
-		my Locale::KeyedText::Translator $translator = Locale::KeyedText.new_translator( 
-			['MyApp::L::', 'MyLib::L::'], @user_lang_prefs );
-
-		show_message( $translator, Locale::KeyedText.new_message( 'MYAPP_HELLO' ) );
-
-		LOOP: {
-			show_message( $translator, Locale::KeyedText.new_message( 'MYAPP_PROMPT' ) );
-			my Str $user_input = $*IN; $user_input.chomp;
-
-			$user_input or last LOOP; # user chose to exit program
-
-			try {
-				my Num $result = MyLib.my_invert( $user_input );
-				show_message( $translator, Locale::KeyedText.new_message( 'MYAPP_RESULT', 
-					{ 'ORIGINAL' => $user_input, 'INVERTED' => $result } ) );
-				CATCH {
-					show_message( $translator, $! ); # input error, detected by library
-				}
-			};
-
-			redo LOOP;
-		}
-
-		show_message( $translator, Locale::KeyedText.new_message( 'MYAPP_GOODBYE' ) );
-	}
-
-	sub show_message( Locale::KeyedText::Translator $translator, Locale::KeyedText::Message $message ) returns Str {
-		my Str $user_text = $translator.translate_message( $message );
-		unless( $user_text ) {
-			print $*ERR "internal error: can't find user text for a message: \n"~
-				"   "~$message.as_string()~"\n"~
-				"   "~$translator.as_string()~"\n";
-			exit;
-		}
-		print $*OUT $user_text~"\n";
-	}
-
-=head2 Content of English language Template file 'MyApp/L/Eng.pm':
-
-	module MyApp::L::Eng;
-
-	my Str %text_strings ::= (
-		'MYAPP_HELLO' => 'Welcome to MyApp.',
-		'MYAPP_GOODBYE' => 'Goodbye!',
-		'MYAPP_PROMPT' => 'Enter a number to be inverted, or press ENTER to quit.',
-		'MYAPP_RESULT' => 'The inverse of "{ORIGINAL}" is "{INVERTED}".',
-	);
-
-	sub get_text_by_key( Str $msg_key ) returns Str { return $text_strings{$msg_key}; }
-
-=head2 Content of French language (rough manual translation) Template file 'MyApp/L/Fre.pm':
-
-	module MyApp::L::Fre;
-
-	my Str %text_strings ::= (
-		'MYAPP_HELLO' => 'Bienvenue allé MyApp.',
-		'MYAPP_GOODBYE' => 'Salut!',
-		'MYAPP_PROMPT' => 'Fournir nombre être inverser, ou appuyer sur ENTER être arrêter.',
-		'MYAPP_RESULT' => 'Renversement "{ORIGINAL}" est "{INVERTED}".',
-	);
-
-	sub get_text_by_key( Str $msg_key ) returns Str { return $text_strings{$msg_key}; }
-
-=head2 Content of alternate text Template file 'MyApp/L/Homer.pm':
-
-	module MyApp::L::Homer;
-
-	my Str %text_strings ::= (
-		'MYAPP_HELLO' => 'Light goes on!',
-		'MYAPP_GOODBYE' => 'Light goes off!',
-		'MYAPP_PROMPT' => 'Give me a county thingy, or push that big button instead.',
-		'MYAPP_RESULT' => 'Turn "{ORIGINAL}" upside down and get "{INVERTED}", not "{ORIGINAL}".',
-		'MYLIB_MYINV_NO_ARG' => 'Why you little ...!',
-		'MYLIB_MYINV_BAD_ARG' => '"{GIVEN_VALUE}" isn\'t a county thingy!',
-		'MYLIB_MYINV_RES_INF' => 'Don\'t you give me a big donut!',
-	);
-
-	sub get_text_by_key( Str $msg_key ) returns Str { return $text_strings{$msg_key}; }
+I<Note that the above example only shows off a few of Locale::KeyedText's
+features; for a larger and more complete example, see the EXAMPLE PROGRAM
+documentation sections further below.>
 
 =head1 DESCRIPTION
 
 Many times during a program's operation, the program (or a module it uses) will
 need to display a message to the user, or generate a message to be shown to the
-user.  Often this is an error message of some kind, but it could also be a
+user.  Sometimes this is an error message of some kind, but it could also be a
 prompt message for interactive systems.
 
+If the program or any of its components are intended for widespread use then it
+needs to account for a variance of needs between its different users, such as
+their preferred language of communication, or their privileges regarding access
+to information details, or their technical skills.  For example, a native French
+or Chinese speaker often prefers to communicate in those languages.  Or, when
+viewing an error message, the application's developer should see more details
+than joe public would.
+
+Alternately, sometimes a program will raise a condition or error that, while
+resembling a message that would be shown to a user, is in fact meant to be
+interpreted by the machine itself and not any human user.  In some situations, 
+a shared program component may raise such a condition, and one application may 
+handle it internally, while another one displays it to the user instead.
+
+Locale::KeyedText provides a simple but effective mechanism for applications and
+modules that empowers single binaries to support N locales or user types
+simultaneously, and that allows any end users to add support for new locales
+easily and without a recompile, often even while the program is executing.
+
+Locale::KeyedText itself is trivially easy to install, since it is written in
+pure Perl and it has no external dependencies of any kind.
+
+Practically speaking, Locale::KeyedText doesn't actually do a lot internally; it
+exists mainly to document a certain localization methodology in an easily
+accessable manner, such that would not be possible if its functionality was
+subsumed into a larger module that would otherwise use it.  Hereafter, if any
+other module or application says that it uses Locale::KeyedText, that is a terse
+way of saying that it subscribes to the localization methodology that is
+described here, and hence provides these benefits to developers and users alike.
+
+B<WHAT FOLLOWS IS MOST OF THE OLD 'DESCRIPTION'; IT WILL BE REWRITTEN/REMOVED 
+WITHIN THE NEXT FEW HOURS.>
+
 Modern programs or database systems often refer to an error condition by an
 internal code which is guaranteed to be unique for a situation, and this is
 mapped to a user-readable message at some point.  For example, Oracle databases
@@ -553,9 +490,9 @@ are not in [a-zA-Z0-9_:].
 =head1 SYNTAX
 
 This class does not export any functions or methods, so you need to call them
-using object notation.  This means using B<Class-E<gt>function()> for functions
-and B<$object-E<gt>method()> for methods.  If you are inheriting this class for
-your own modules, then that often means something like B<$self-E<gt>method()>.  
+using object notation.  This means using B<Class.function()> for functions and
+B<$object.method()> for methods.  If you are inheriting this class for your own
+modules, then that often means something like B<$self.method()>.
 
 =head1 CONSTRUCTOR WRAPPER FUNCTIONS
 
@@ -677,7 +614,123 @@ This method returns a stringified version of this object which is suitable for
 debugging purposes (such as to test that the object's contents look good at a
 glance); no property values are escaped and you shouldn't try to extract them.
 
-=head1 AN ALTERNATIVE TO SEPARATED TEMPLATE FILES
+=head1 EXAMPLE PROGRAM WITH ENTIRELY SEPARATED TEMPLATE MODULE FILES
+
+The following demonstrates a simple library and a simple program that uses it;
+both are N-multi-lingual and ship with English and French support files.  While 
+there is no support file specific to the library for a certain third language, 
+the one with the program also adds support to the library.
+
+Content of shared library file 'MyLib.pm':
+
+	module MyLib;
+
+	require Locale::KeyedText;
+
+	sub my_invert( Str $number ) returns Num {
+		$number.defined or throw Locale::KeyedText.new_message( 'MYLIB_MYINV_NO_ARG' );
+		$number ~~ m/\d/ or throw Locale::KeyedText.new_message( 
+			'MYLIB_MYINV_BAD_ARG', { 'GIVEN_VALUE' => $number } );
+		$number == 0 and throw Locale::KeyedText.new_message( 'MYLIB_MYINV_RES_INF' );
+		return 1 / $number;
+	}
+
+Content of English language Template file 'MyLib/L/Eng.pm':
+
+	module MyLib::L::Eng;
+	my Str %text_strings ::= (
+		'MYLIB_MYINV_NO_ARG' => 'my_invert(): argument NUMBER is missing',
+		'MYLIB_MYINV_BAD_ARG' => 'my_invert(): argument NUMBER is not a number, it is "{GIVEN_VALUE}"',
+		'MYLIB_MYINV_RES_INF' => 'my_invert(): result is infinite because argument NUMBER is zero',
+	);
+	sub get_text_by_key( Str $msg_key ) returns Str { return %text_strings{$msg_key}; }
+
+Content of French language (rough manual translation) Template file 'MyLib/L/Fre.pm':
+
+	module MyLib::L::Fre;
+	my Str %text_strings ::= (
+		'MYLIB_MYINV_NO_ARG' => 'my_invert(): paramètre NUMBER est manquant',
+		'MYLIB_MYINV_BAD_ARG' => 'my_invert(): paramètre NUMBER est ne nombre, il est "{GIVEN_VALUE}"',
+		'MYLIB_MYINV_RES_INF' => 'my_invert(): aboutir a est infini parce que paramètre NUMBER est zero',
+	);
+	sub get_text_by_key( Str $msg_key ) returns Str { return %text_strings{$msg_key}; }
+
+Content of main program 'MyApp.pl':
+
+	require MyLib;
+	require Locale::KeyedText;
+
+	main( grep { $_ ~~ m/^<[a-zA-Z]>+$/ } @*ARGS ); # user indicates language as command line argument
+
+	sub main( Str ?@user_lang_prefs = 'Eng' ) {
+		my Locale::KeyedText::Translator $translator = Locale::KeyedText.new_translator( 
+			['MyApp::L::', 'MyLib::L::'], @user_lang_prefs );
+		show_message( $translator, Locale::KeyedText.new_message( 'MYAPP_HELLO' ) );
+		LOOP: {
+			show_message( $translator, Locale::KeyedText.new_message( 'MYAPP_PROMPT' ) );
+			my Str $user_input = $*IN; $user_input.chomp;
+			$user_input or last LOOP; # user chose to exit program
+			try {
+				my Num $result = MyLib.my_invert( $user_input );
+				show_message( $translator, Locale::KeyedText.new_message( 'MYAPP_RESULT', 
+					{ 'ORIGINAL' => $user_input, 'INVERTED' => $result } ) );
+				CATCH {
+					show_message( $translator, $! ); # input error, detected by library
+				}
+			};
+			redo LOOP;
+		}
+		show_message( $translator, Locale::KeyedText.new_message( 'MYAPP_GOODBYE' ) );
+	}
+
+	sub show_message( Locale::KeyedText::Translator $translator, Locale::KeyedText::Message $message ) returns Str {
+		my Str $user_text = $translator.translate_message( $message );
+		unless( $user_text ) {
+			print $*ERR "internal error: can't find user text for a message: \n"~
+				"   "~$message.as_string()~"\n"~
+				"   "~$translator.as_string()~"\n";
+			exit;
+		}
+		print $*OUT $user_text~"\n";
+	}
+
+Content of English language Template file 'MyApp/L/Eng.pm':
+
+	module MyApp::L::Eng;
+	my Str %text_strings ::= (
+		'MYAPP_HELLO' => 'Welcome to MyApp.',
+		'MYAPP_GOODBYE' => 'Goodbye!',
+		'MYAPP_PROMPT' => 'Enter a number to be inverted, or press ENTER to quit.',
+		'MYAPP_RESULT' => 'The inverse of "{ORIGINAL}" is "{INVERTED}".',
+	);
+	sub get_text_by_key( Str $msg_key ) returns Str { return %text_strings{$msg_key}; }
+
+Content of French language (rough manual translation) Template file 'MyApp/L/Fre.pm':
+
+	module MyApp::L::Fre;
+	my Str %text_strings ::= (
+		'MYAPP_HELLO' => 'Bienvenue allé MyApp.',
+		'MYAPP_GOODBYE' => 'Salut!',
+		'MYAPP_PROMPT' => 'Fournir nombre être inverser, ou appuyer sur ENTER être arrêter.',
+		'MYAPP_RESULT' => 'Renversement "{ORIGINAL}" est "{INVERTED}".',
+	);
+	sub get_text_by_key( Str $msg_key ) returns Str { return %text_strings{$msg_key}; }
+
+Content of alternate text Template file 'MyApp/L/Homer.pm':
+
+	module MyApp::L::Homer;
+	my Str %text_strings ::= (
+		'MYAPP_HELLO' => 'Light goes on!',
+		'MYAPP_GOODBYE' => 'Light goes off!',
+		'MYAPP_PROMPT' => 'Give me a county thingy, or push that big button instead.',
+		'MYAPP_RESULT' => 'Turn "{ORIGINAL}" upside down and get "{INVERTED}", not "{ORIGINAL}".',
+		'MYLIB_MYINV_NO_ARG' => 'Why you little ...!',
+		'MYLIB_MYINV_BAD_ARG' => '"{GIVEN_VALUE}" isn\'t a county thingy!',
+		'MYLIB_MYINV_RES_INF' => 'Don\'t you give me a big donut!',
+	);
+	sub get_text_by_key( Str $msg_key ) returns Str { return %text_strings{$msg_key}; }
+
+=head1 ALTERNATE EXAMPLE PROGRAM WITH MOSTLY INTEGRATED TEMPLATE MODULES
 
 A feature extension to Locale::KeyedText allows you to store your Template
 class packages inside the same files as your other program code, rather than
@@ -697,7 +750,7 @@ Template class packages for MyLib and MyApp embedded in the code files rather
 than being separate; this example totals 3 files instead of the old 7 files.  
 Actually, it shows both methods together, with 4 embedded, 1 separate.
 
-=head2 Content of shared library file 'MyLib.pm':
+Content of shared library file 'MyLib.pm':
 
 	module MyLib {
 		require Locale::KeyedText;
@@ -716,8 +769,7 @@ Actually, it shows both methods together, with 4 embedded, 1 separate.
 			'MYLIB_MYINV_BAD_ARG' => 'my_invert(): argument NUMBER is not a number, it is "{GIVEN_VALUE}"',
 			'MYLIB_MYINV_RES_INF' => 'my_invert(): result is infinite because argument NUMBER is zero',
 		);
-
-		sub get_text_by_key( Str $msg_key ) returns Str { return $text_strings{$msg_key}; }
+		sub get_text_by_key( Str $msg_key ) returns Str { return %text_strings{$msg_key}; }
 	}
 
 	module MyLib::L::Fre {
@@ -726,10 +778,10 @@ Actually, it shows both methods together, with 4 embedded, 1 separate.
 			'MYLIB_MYINV_BAD_ARG' => 'my_invert(): paramètre NUMBER est ne nombre, il est "{GIVEN_VALUE}"',
 			'MYLIB_MYINV_RES_INF' => 'my_invert(): aboutir a est infini parce que paramètre NUMBER est zero',
 		);
-		sub get_text_by_key( Str $msg_key ) returns Str { return $text_strings{$msg_key}; }
+		sub get_text_by_key( Str $msg_key ) returns Str { return %text_strings{$msg_key}; }
 	}
 
-=head2 Content of main program 'MyApp.pl':
+Content of main program 'MyApp.pl':
 
 	require MyLib;
 	require Locale::KeyedText;
@@ -739,15 +791,11 @@ Actually, it shows both methods together, with 4 embedded, 1 separate.
 	sub main( Str ?@user_lang_prefs = 'Eng' ) {
 		my Locale::KeyedText::Translator $translator = Locale::KeyedText.new_translator( 
 			['MyApp::L::', 'MyLib::L::'], @user_lang_prefs );
-
 		show_message( $translator, Locale::KeyedText.new_message( 'MYAPP_HELLO' ) );
-
 		LOOP: {
 			show_message( $translator, Locale::KeyedText.new_message( 'MYAPP_PROMPT' ) );
 			my Str $user_input = $*IN; $user_input.chomp;
-
 			$user_input or last LOOP; # user chose to exit program
-
 			try {
 				my Num $result = MyLib.my_invert( $user_input );
 				show_message( $translator, Locale::KeyedText.new_message( 'MYAPP_RESULT', 
@@ -756,10 +804,8 @@ Actually, it shows both methods together, with 4 embedded, 1 separate.
 					show_message( $translator, $! ); # input error, detected by library
 				}
 			};
-
 			redo LOOP;
 		}
-
 		show_message( $translator, Locale::KeyedText.new_message( 'MYAPP_GOODBYE' ) );
 	}
 
@@ -781,7 +827,7 @@ Actually, it shows both methods together, with 4 embedded, 1 separate.
 			'MYAPP_PROMPT' => 'Enter a number to be inverted, or press ENTER to quit.',
 			'MYAPP_RESULT' => 'The inverse of "{ORIGINAL}" is "{INVERTED}".',
 		);
-		sub get_text_by_key( Str $msg_key ) returns Str { return $text_strings{$msg_key}; }
+		sub get_text_by_key( Str $msg_key ) returns Str { return %text_strings{$msg_key}; }
 	}
 
 	module MyApp::L::Fre {
@@ -791,13 +837,12 @@ Actually, it shows both methods together, with 4 embedded, 1 separate.
 			'MYAPP_PROMPT' => 'Fournir nombre être inverser, ou appuyer sur ENTER être arrêter.',
 			'MYAPP_RESULT' => 'Renversement "{ORIGINAL}" est "{INVERTED}".',
 		);
-		sub get_text_by_key( Str $msg_key ) returns Str { return $text_strings{$msg_key}; }
+		sub get_text_by_key( Str $msg_key ) returns Str { return %text_strings{$msg_key}; }
 	}
 
-=head2 Content of alternate text Template file 'MyApp/L/Homer.pm':
+Content of alternate text Template file 'MyApp/L/Homer.pm':
 
 	module MyApp::L::Homer;
-
 	my Str %text_strings ::= (
 		'MYAPP_HELLO' => 'Light goes on!',
 		'MYAPP_GOODBYE' => 'Light goes off!',
@@ -807,8 +852,7 @@ Actually, it shows both methods together, with 4 embedded, 1 separate.
 		'MYLIB_MYINV_BAD_ARG' => '"{GIVEN_VALUE}" isn\'t a county thingy!',
 		'MYLIB_MYINV_RES_INF' => 'Don\'t you give me a big donut!',
 	);
-
-	sub get_text_by_key( Str $msg_key ) returns Str { return $text_strings{$msg_key}; }
+	sub get_text_by_key( Str $msg_key ) returns Str { return %text_strings{$msg_key}; }
 
 =head1 BUGS
 
@@ -842,6 +886,20 @@ require external files itself to resolve them, leading to recursive complexity.
 Suggestions for an alternate "proper" solution are welcome; meanwhile, the
 current solution seems best to me.
 
+=head1 CREDITS
+
+Besides myself as the creator ...
+
+* 2004.07.26 - Thanks to Jason Martin (jhmartin@toger.us) for suggesting a
+feature, along with providing sample usage and patch code, that supports
+embedding of Template class packages in the same files as program code, rather
+than requiring separate files.
+
+* 2005.03.21 - Thanks to Stevan Little (stevan@iinteractive.com) for feedback
+towards improving this module's documentation, particularly towards using a much
+shorter SYNOPSIS, so that it is easier for newcomers to understand the module at
+a glance, and not be intimidated by large amounts of detailed information.
+
 =head1 SEE ALSO
 
 These Perl 5 modules: L<Locale::Maketext>, L<Locale::gettext>,
