@@ -1,4 +1,4 @@
-module Sample::Module;
+module Sample::Module-0.0.1;
 use v6;
 
 sub greeting(Str $name) returns Str is export {
