@@ -1,4 +1,5 @@
 {-# OPTIONS -fglasgow-exts -O #-}
+{-# OPTIONS -#include "UnicodeC.h" #-}
 
 {-
     Lexical analyzer.
@@ -90,6 +91,9 @@ symbol s
     aheadSym '-' '>' = False -- XXX hardcode
     aheadSym '!' '~' = False -- XXX hardcode
     aheadSym x   '=' = not (x `elem` "!~+-*&/|.")
+    aheadSym '?' y   = not (y `elem` "&|^?")
+    aheadSym '+' y   = not (y `elem` "&|^<>+")
+    aheadSym '~' y   = not (y `elem` "&|^<>~")
     aheadSym x   y   = y `elem` ";!" || x /= y
 
 stringLiteral = singleQuoted
