@@ -73,7 +73,7 @@ instance Pretty Val where
     format (VNum x) = if x == 1/0 then text "Inf" else text $ show x
     format (VInt x) = integer x
     format (VStr x) = text $ "'" ++ encodeUTF8 (concatMap quoted x) ++ "'"
-    format (VRat x) = double $ ((fromIntegral $ numerator x) / (fromIntegral $ denominator x) :: Double)
+    format (VRat x) = double $ ratToNum x
     format (VComplex x) = text $ show x
     format (VControl x) = text $ show x
     format (VRef (VList x))
@@ -89,20 +89,24 @@ instance Pretty Val where
     format (VBlock _) = text "{...}"
     format (VError x y) = hang (text "*** Error:" <+> text x) defaultIndent (text "at" <+> format y)
     format (VArray (MkArray x)) = format (VList x)
-    format (VHash (MkHash x)) = braces $ (joinList $ text ", ") (map format $ fmToList x)
-    
+    format (VHash (MkHash x)) = braces $ (joinList $ text ", ") $
+        [ format (VStr k, v) | (k, v) <- fmToList x ]
     format (VHandle x) = text $ show x
     format (MVal v) = text $ unsafePerformIO $ do
         val <- readIORef v
         return $ pretty val
     format (VThunk _) = text $ "{thunk}"
     format (VRule _) = text $ "{rule}"
+    format (VSubst _) = text $ "{subst}"
     format VUndef = text $ "undef"
 
 quoted '\'' = "\\'"
 quoted '\\' = "\\\\"
 quoted x = [x]
 
+ratToNum :: VRat -> VNum
+ratToNum x = (fromIntegral $ numerator x) / (fromIntegral $ denominator x)
+
 doubleBraces :: Doc -> Doc
 doubleBraces x = vcat [ (lbrace <> lbrace), nest defaultIndent x, rbrace <> rbrace]
 
